/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtGui>
#include <QtOpenGL>

#include "glwidget.h"
#include <array>

GLWidget::GLWidget(QWidget *parent, QGLWidget *shareWidget)
    : QGLWidget(parent, shareWidget)
{
    clearColor = Qt::black;
    xRot = 0;
    yRot = 0;
    zRot = 0;
	xTrans = val_glwidget_init_trans_x;
    yTrans = val_glwidget_init_trans_y;
    zTrans = val_glwidget_init_trans_z;

	isInit = false;
	enableSelectionBox = true;
	drawSelectionBox = false;
	selectionboxchanged = false;
	dragstarted = false;
	abovezero=false;

#ifdef QT_OPENGL_ES_2
    program = 0;
#endif
}

GLWidget::~GLWidget()
{
}

QSize GLWidget::minimumSizeHint() const
{
    return QSize(1000, 600);
}

QSize GLWidget::sizeHint() const
{
    return QSize(1000, 600);
}

void GLWidget::rotateBy(int xAngle, int yAngle, int zAngle)
{
    xRot += xAngle;
    yRot += yAngle;
    zRot += zAngle;
    updateGL();
}

void GLWidget::translateBy(double dx, double dy, double dz)
{
    xTrans += dx;
    yTrans -= dy;
	zTrans = min(max(zTrans+dz,-299), -0.0002);	//should stay inside the view frustrum
    updateGL();
}

void GLWidget::setClearColor(const QColor &color)
{
    clearColor = color;
    updateGL();
}

//load the texture specified to a user and create the base image rect using its startup function
void GLWidget::loadTexture(QString fname)
{
	imagerect = new imageRect();
	imagerect->startUp(fname,val_rect_start_width,val_rect_start_height);

	texture = bindTexture(*imagerect->getTex(), GL_TEXTURE_2D);

	isInit=true;
	
	updateGL();
	emit updateNumberOfComponents(imagerect->getNumberOfComponents());
	setMouseTracking(true);		//this way the mouse is always tracked, and not only on-press
}

void GLWidget::initializeGL()
{


    glEnable(GL_DEPTH_TEST);
    glDisable(GL_CULL_FACE);
#ifndef QT_OPENGL_ES_2
    glEnable(GL_TEXTURE_2D);
#endif

#ifdef QT_OPENGL_ES_2

#define PROGRAM_VERTEX_ATTRIBUTE 0
#define PROGRAM_TEXCOORD_ATTRIBUTE 1

    QGLShader *vshader = new QGLShader(QGLShader::Vertex, this);
    const char *vsrc =
        "attribute highp vec4 vertex;\n"
        "attribute mediump vec4 texCoord;\n"
        "varying mediump vec4 texc;\n"
        "uniform mediump mat4 matrix;\n"
        "void main(void)\n"
        "{\n"
        "    gl_Position = matrix * vertex;\n"
        "    texc = texCoord;\n"
        "}\n";
    vshader->compileSourceCode(vsrc);

    QGLShader *fshader = new QGLShader(QGLShader::Fragment, this);
    const char *fsrc =
        "uniform sampler2D texture;\n"
        "varying mediump vec4 texc;\n"
        "void main(void)\n"
        "{\n"
        "    gl_FragColor = texture2D(texture, texc.st);\n"
        "}\n";
    fshader->compileSourceCode(fsrc);

    program = new QGLShaderProgram(this);
    program->addShader(vshader);
    program->addShader(fshader);
    program->bindAttributeLocation("vertex", PROGRAM_VERTEX_ATTRIBUTE);
    program->bindAttributeLocation("texCoord", PROGRAM_TEXCOORD_ATTRIBUTE);
    program->link();

    program->bind();
    program->setUniformValue("texture", 0);

#endif
}

void GLWidget::paintGL()
{


	if(!isInit)
	{ return; }

	updateStatus();
	emit updateNumberOfComponents(imagerect->getNumberOfComponents());

    qglClearColor(clearColor);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

#if !defined(QT_OPENGL_ES_2)

    glLoadIdentity();
	glTranslated(xTrans, yTrans, zTrans);
    glRotated(xRot / 16.0f, 1.0f, 0.0f, 0.0f);
    glRotated(yRot / 16.0f, 0.0f, 1.0f, 0.0f);
    glRotated(zRot / 16.0f, 0.0f, 0.0f, 1.0f);


	glVertexPointer(3, GL_DOUBLE, 0, imagerect->getPositions());
	glTexCoordPointer(2, GL_DOUBLE, 0, imagerect->getTexCoords());
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);

#else

    QMatrix4x4 m;
    m.ortho(-0.5f, +0.5f, +0.5f, -0.5f, 4.0f, 15.0f);
    m.translate(0.0f, 0.0f, -10.0f);
    m.rotate(xRot / 16.0f, 1.0f, 0.0f, 0.0f);
    m.rotate(yRot / 16.0f, 0.0f, 1.0f, 0.0f);
    m.rotate(zRot / 16.0f, 0.0f, 0.0f, 1.0f);

    program->setUniformValue("matrix", m);
    program->enableAttributeArray(PROGRAM_VERTEX_ATTRIBUTE);
    program->enableAttributeArray(PROGRAM_TEXCOORD_ATTRIBUTE);
    program->setAttributeArray
        (PROGRAM_VERTEX_ATTRIBUTE, vertices.constData());
    program->setAttributeArray
        (PROGRAM_TEXCOORD_ATTRIBUTE, texCoords.constData());

#endif

    glBindTexture(GL_TEXTURE_2D, texture);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, (int)imagerect->getVertexCount()/3);		// /3 because this actually holds the count of floats (dirty)


	paintSelectionBox();

	
}

void GLWidget::paintSelectionBox()
{
	//draw selection rectangle
	//fixed function, deprecated stuff, do not tamper with this

	if(enableSelectionBox && drawSelectionBox)
	{
		MyRect box = imagerect->getSelectionBox();

	//glMatrixMode(GL_MODELVIEW);
	//glLoadIdentity();
	//glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();
	//gluOrtho2D(-1, 1, -1, 1);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	//glPushMatrix();
	glBegin(GL_QUADS);
	glColor4f(MyGL_SELECTIONCOLOR_r,MyGL_SELECTIONCOLOR_g,MyGL_SELECTIONCOLOR_b,MyGL_SELECTIONCOLOR_transparency);
	glVertex2f(box.left, box.bottom);
	glColor4f(MyGL_SELECTIONCOLOR_r,MyGL_SELECTIONCOLOR_g,MyGL_SELECTIONCOLOR_b,MyGL_SELECTIONCOLOR_transparency);
	glVertex2f(box.left, box.top);
	glColor4f(MyGL_SELECTIONCOLOR_r,MyGL_SELECTIONCOLOR_g,MyGL_SELECTIONCOLOR_b,MyGL_SELECTIONCOLOR_transparency);
	glVertex2f(box.right, box.top);
	glColor4f(MyGL_SELECTIONCOLOR_r,MyGL_SELECTIONCOLOR_g,MyGL_SELECTIONCOLOR_b,MyGL_SELECTIONCOLOR_transparency);
	glVertex2f(box.right, box.bottom);
	glEnd();
	//glFlush();

	glPopMatrix();
	glDisable(GL_BLEND);
	//glEnable(GL_PROJECTION);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_CULL_FACE);
	glColor4f(1,1,1,1);

	
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(30, 4/3,0.0001,300);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslated(xTrans, yTrans, zTrans);
    glRotated(xRot / 16.0f, 1.0f, 0.0f, 0.0f);
    glRotated(yRot / 16.0f, 0.0f, 1.0f, 0.0f);
    glRotated(zRot / 16.0f, 0.0f, 0.0f, 1.0f);

	}//if enable selection box
}

void GLWidget::resizeGL(int width, int height)
{
    int side = qMin(width, height);
    glViewport(0, 0 , width, height);

#if !defined(QT_OPENGL_ES_2)
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
#ifndef QT_OPENGL_ES
	gluPerspective(30, 4/3,0.0001,300);
#else
    glOrthof(-0.5, +0.5, +0.5, -0.5, 4.0, 15.0);
#endif
    glMatrixMode(GL_MODELVIEW);
#endif
}

void GLWidget::mousePressEvent(QMouseEvent *event){
    lastPos = event->pos();
	startPos = event->pos();

	if(!isInit)
	{
		return;
	}

	//do updateOnClick and save the click position for updateOnDrag
	if(event->buttons() & Qt::LeftButton)
	{
		leftbuttonclicked=true;

		std::array<GLdouble, 16> projection;
		std::array<GLdouble, 16> modelview;
		double bottomleftx,bottomlefty,posz,toprightx,toprighty;

		GLint viewport[4];
		glGetIntegerv(GL_VIEWPORT, viewport);

		glGetDoublev(GL_PROJECTION_MATRIX, projection.data());
		glGetDoublev(GL_MODELVIEW_MATRIX, modelview.data());

		gluProject(this->imagerect->getBoundingBox().left, this->imagerect->getBoundingBox().bottom, 0,
			modelview.data(), projection.data(),viewport,
			&bottomleftx, &bottomlefty, &posz);		//returns pixel coordinates with 0,0 as bottom left corner of gl draw window

		gluProject(this->imagerect->getBoundingBox().right, this->imagerect->getBoundingBox().top, 0,
			modelview.data(), projection.data(),viewport,
			&toprightx, &toprighty, &posz);	

		float curx = event->x();
		float cury = this->height() - event->y(); //transform event.y because it has origin at top left corner instead bottom left

		if((curx > bottomleftx && 
			cury > bottomlefty)&&
			(curx < toprightx && 
			cury < toprighty))
		{	//the cursor is inside of our rect
			drawSelectionBox = true;
			

			//normalize x to the rect's (0,1) space
			double normx = (curx - bottomleftx) / (toprightx - bottomleftx);
			double normy = (cury - bottomlefty) / (toprighty - bottomlefty);

			lastnormx = normx;
			lastnormy = normy;

			lastdiffx = 0;
			lastdiffy = 0;

			imagerect->updateOnClick(normx,normy);
			imagerect->updateSelection(normx,normy);
			emit idChanged(imagerect->getCurrentSelectionId());

			imagerect->fixSelecction();

			updateGL();
		}


	}
}

void GLWidget::mouseMoveEvent(QMouseEvent *event)
{
	if(!isInit)
	{
		return;
	}

    int dx = event->x() - lastPos.x();
    int dy = event->y() - lastPos.y();

	double dxScaled = (double)dx*-zTrans/this->width();
	double dyScaled = (double)dy*-zTrans/this->height();


	
	//do right button actions
	if (event->buttons() & Qt::RightButton) {
        translateBy(dxScaled, dyScaled, 0.0);
    }
    lastPos = event->pos();

	//do mouse move actions
	//update selection: first check if mouse is inside rect, if yes then pass the normalized coords on to the rect




	//project our rect to image space and find the normalized mouse pointer location
	std::array<GLdouble, 16> projection;
	std::array<GLdouble, 16> modelview;
	double bottomleftx,bottomlefty,posz,toprightx,toprighty;

	GLint viewport[4];
	glGetIntegerv(GL_VIEWPORT, viewport);

	glGetDoublev(GL_PROJECTION_MATRIX, projection.data());
	glGetDoublev(GL_MODELVIEW_MATRIX, modelview.data());

	gluProject(this->imagerect->getBoundingBox().left, this->imagerect->getBoundingBox().bottom, 0,
    modelview.data(), projection.data(),viewport,
    &bottomleftx, &bottomlefty, &posz);		//returns pixel coordinates with 0,0 as bottom left corner of gl draw window

	gluProject(this->imagerect->getBoundingBox().right, this->imagerect->getBoundingBox().top, 0,
    modelview.data(), projection.data(),viewport,
    &toprightx, &toprighty, &posz);	

	double curx = event->x();
	double cury = this->height() - event->y(); //transform event.y because it has origin at top left corner instead bottom left

	//normalize x to the rect's (0,1) space
	double normx = (curx - bottomleftx) / (toprightx - bottomleftx);
	double normy = (cury - bottomlefty) / (toprighty - bottomlefty);

	if((curx > bottomleftx && 
		cury > bottomlefty)&&
		(curx < toprightx && 
		cury < toprighty))
	{	//the cursor is inside of our rect
		drawSelectionBox = true;



		//pass the coordinates on to the rect to update its selection
		//when painting, retrieve its updated selection box
		imagerect->updateSelection(normx,normy);
		emit idChanged(imagerect->getCurrentSelectionId());



		//always have to update inside the rect
		updateGL();
	}
	else
	{
		drawSelectionBox = false;
		emit idChanged(-1);
	}
	if(drawSelectionBox != selectionboxchanged)
	{
		updateGL();		//update GL only if visibility has changed
	}
	selectionboxchanged = drawSelectionBox;

	//do left button actions
	if (event->buttons() & Qt::LeftButton) {

		if(leftbuttonclicked)
		{

			double diffx = normx - lastnormx;
			double diffy = normy - lastnormy;

			imagerect->updateOnDrag(lastnormx, lastnormy, diffx,diffy);

			/*if(lastdiffy == 0)		//this entire if-block is only for the GUI, "dragging" the mouse up and down should not produce erroneous commands
			{
				if(diffy > 0)
				{
					abovezero=true;
				}
				else
				{
					abovezero=false;
				}
			}
			else
			{
				if(abovezero && (diffy-lastdiffy<0))
				{
					imagerect->updateOnDrag(lastnormx, lastnormy, 0,0);
				}
				else if (!abovezero && (diffy-lastdiffy>0))
				{
					imagerect->updateOnDrag(lastnormx, lastnormy, 0,0);
				}
				else
				{
					imagerect->updateOnDrag(lastnormx, lastnormy, diffx,diffy);
				}

			}*/

			//cout<<"mouse dragged for "<<diffx<<" and "<<diffy<<std::endl;


			lastdiffx = diffx;
			lastdiffy = diffy;

			updateGL();
		}
	} 





}

void GLWidget::mouseReleaseEvent(QMouseEvent * /* event */)
{
	leftbuttonclicked=false;
	dragstarted = false;
	imagerect->releaseSelection();
    emit clicked();
}

void GLWidget::wheelEvent(QWheelEvent *event){
	double dzScaled = (double)event->delta()*zTrans/this->width();
	translateBy(0,0,dzScaled);
}

void GLWidget::changeGUIStatus(char* message)
{
	emit statusChanged(QString(message));
}

void GLWidget::updateStatus()
{
	changeGUIStatus(imagerect->getStatusMessage());
}