/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef GLWIDGET_H
#define GLWIDGET_H

#include <QtGui>
#include <QGLWidget>
#include <iostream>
#include <GLU.H>
#include <plane.h>
#include <imageRect.h>
#include "config.h"

using namespace std;

class QGLShaderProgram;

class GLWidget : public QGLWidget
{
    Q_OBJECT

public:
	GLWidget(QWidget *parent = 0, QGLWidget *shareWidget = 0);
    ~GLWidget();

    QSize minimumSizeHint() const;
    QSize sizeHint() const;
    void rotateBy(int xAngle, int yAngle, int zAngle);
	void translateBy(double dx, double dy, double dz);	//pixels
    void setClearColor(const QColor &color);
	void setSelectionBox(bool enabled) {enableSelectionBox=enabled; updateGL();}
	void loadTexture(QString fname);
	void resetComponents() {if(!isInit){return;}  this->imagerect->resetComponents(); statusChanged(text_status_reset); emit updateNumberOfComponents(imagerect->getNumberOfComponents());}
	void changeGUIStatus(char* message);
	void setstartwidth(double width) { imagerect->setstartwidth(width); }

signals:
    void clicked();
	void idChanged(int id);
	void statusChanged(QString message);
	void updateNumberOfComponents(int components);

protected:
    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
	virtual void wheelEvent(QWheelEvent *event);
	void paintSelectionBox();

private:
	void updateStatus();

    QColor clearColor;
    QPoint lastPos;
	QPoint startPos;
    int xRot;
    int yRot;
    int zRot;
	double xTrans;
    double yTrans;
    double zTrans;
    GLuint texture;

	double lastnormx;
	double lastnormy;
	double lastdiffx;
	double lastdiffy;
	bool abovezero;

	imageRect *imagerect;

	bool isInit;
	bool enableSelectionBox;
	bool drawSelectionBox;
	bool selectionboxchanged;

	bool leftbuttonclicked;
	bool dragstarted;
#ifdef QT_OPENGL_ES_2
    QGLShaderProgram *program;
#endif
};

#endif
