#include "imageRect.h"

double imageRect::getWorldPosition(double xNorm)
{
	return myBoundingBox.left + xNorm*(myBoundingBox.right-myBoundingBox.left);
}


//initialize the geometry: create one plane
void imageRect::startUp( QString filename, double width, double height )
{
	tex = new QPixmap(filename);

	//set the basic bounding box for this object 
	myBoundingBox = MyRect();
	myBoundingBox.top = height/2;
	myBoundingBox.bottom = -height/2;
	myBoundingBox.left = -width/2;
	myBoundingBox.right = width/2;
	initBoundingBox = myBoundingBox;
	initRange = myBoundingBox.right-myBoundingBox.left;

	//set coordinates for the first child plane
	componentPlane* baseplane = new componentPlane(runningId);
	runningId++;	//update the running Id
	MyRect baserect;
	baserect.bottom = 0;
	baserect.top = 1;
	baserect.left = 0;
	baserect.right = 1;
	baseplane->startUp(baserect,baserect);

	components.push_back(baseplane);

	calcTexCoordsTotal();

	setstartwidth(Config::val_fold_startval_width);

	newStatus(text_status_initrect);

	initialized = true;
    
}

	//return vertices: iterate over all children and get their positions

	double* imageRect::getPositions()
	{
		float dbx = abs(myBoundingBox.right - myBoundingBox.left);	//use the rect's length and width to transform component coordinates
		float dby = abs(myBoundingBox.top - myBoundingBox.bottom);

		int numVertices=0;	
		double* temp = new double[sys_arrayinit];
		std::list<rectComponent*>::iterator it = components.begin();
		for(int i=0; i<components.size(); i++)	//for all children, get their position components...
		{
			std::vector<std::vector<double>> currentpos = (*it)->getPositions();	//get one child's normalized x,y,z components
			int countOfFloats=currentpos.at(0).size()*3;	//first child's x component * 3 is the number of floats
			

			int thisposition=0;
			for(int j=numVertices; j<numVertices+countOfFloats; j++)	//... and append them to our current result
			{
				//transform the normalized coordinates using this rect's bounding box
				temp[j]=myBoundingBox.left + currentpos.at(0).at(thisposition) * dbx;	//x-component
				temp[++j]=myBoundingBox.bottom + currentpos.at(1).at(thisposition) * dby;	//y-component
				temp[++j]=currentpos.at(2).at(thisposition);	//z-component

				++thisposition;
			}
			numVertices+=countOfFloats;
			it++;
		}
		
		double* result = new double[numVertices];

		//update vertexcount
		vertexCount = (int)numVertices;

		for(int i=0; i<numVertices; i++)		//iterate over all found child vertices, add them to the result array
		{
			result[i] = temp[i];
		}

		//delete temp;

		return result;
	}


double* imageRect::getTexCoords()
{
		int numTexcoords=0;	
		double* temp = new double[sys_arrayinit];
		std::list<rectComponent*>::iterator it = components.begin();
		for(int i=0; i<components.size(); i++)	//for all children, get their position components...
		{
			std::vector<double> currenttex = (*it)->getTexCoords();
			int countOfFloats=currenttex.size();
			

			int thisposition=0;
			for(int j=numTexcoords; j<numTexcoords+countOfFloats; j++)	//... and append them to our current result
			{
				temp[j]=currenttex.at(thisposition);
				++thisposition;
			}
			numTexcoords+=countOfFloats;
			it++;
		}
		
		double* result = new double[numTexcoords];

		for(int i=0; i<numTexcoords; i++)		//iterate over all found child vertices, add them to the result array
		{
			result[i] = temp[i];
		}

		//delete temp;

		return result;

}



//handle the box for the selection
//update the current selection using the current 2D (normalized) "rect"-space coordinates
//actions: set new selection, keep old selection or set "no selection"
void imageRect::updateSelection(double x, double y)
{
	int tempid=0;

	//only regard x coordinate since we have only left-to-right folds
	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size(); i++)
	{
		MyRect bbox = (*it)->getBoundingBox();

		//since the components are ordered left-to-right, just check
		//for the last component whose left border isnt larger than x
		if(bbox.left > x)
		{
			it--;
			tempid=(*it)->getId();
			break;
		}

		//if not, return last component
		tempid=(*it)->getId();
		it++;
	}

	selection = tempid;


	//update selection box
	rectComponent* currentcomp = getComponentById(selection);
	currentSelectionBox.left = currentcomp->getBoundingBox().left;
	currentSelectionBox.top = currentcomp->getBoundingBox().top;
	currentSelectionBox.right = currentcomp->getBoundingBox().right;
	currentSelectionBox.bottom = currentcomp->getBoundingBox().bottom;
}


rectComponent* imageRect::getComponentById(int id)
{
	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size(); i++)
	{
		if( (*it)->getId() == id )
		{
			return (*it);
		}
		it++;
	}
	return 0;
}

rectComponent* imageRect::getComponentByPosition(double x, double y)
{
	rectComponent* res = 0;

	//only regard x coordinate since components are ordered left-to-right
	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size(); i++)
	{
		MyRect bbox = (*it)->getBoundingBox();

		//since the components are ordered left-to-right, just check
		//for the last component whose left border isnt larger than x
		if(bbox.left > x)
		{
			it--;
			res=(*it);
			break;
		}

		//if not, return last component
		res=(*it);
		it++;
	}

	return res;
}


//return selection box in world coordinates = transformed bounding box of current selected component
MyRect imageRect::getSelectionBox()
{
	MyRect selectionBox;

	double dbx = abs(myBoundingBox.right - myBoundingBox.left);
	double dby = abs(myBoundingBox.top - myBoundingBox.bottom);

	//return current selection's bounding box
	selectionBox.top = myBoundingBox.bottom + currentSelectionBox.top * dby;
	selectionBox.right = myBoundingBox.left + currentSelectionBox.right * dbx ;
	selectionBox.bottom = myBoundingBox.bottom + currentSelectionBox.bottom * dby;
	selectionBox.left = myBoundingBox.left + currentSelectionBox.left * dbx ;

	return selectionBox;
}

//update the component on this position: 
//if plane: insert new fold (and new plane)
//if fold: edit fold or remove fold
void imageRect::updateOnClick(double x, double y)
{
	rectComponent* currentcomponent = getComponentByPosition(x,y);
	
	//IF PLANE
	if(currentcomponent->getType() == MyCompType::OURPLANE)
	{
		//create a new fold with center at x, reduce the size of the two neighbouring planes by width/2

		double newcenter = x;
		double newwidth = val_fold_start_width;

		rectComponent* leftneighbour = currentcomponent;
		MyRect leftneighboursizebox = currentcomponent->getBoundingBox();
		MyRect leftneighbourtexbox = currentcomponent->getTextureBoundingBox();

		double cfac1 = (newcenter-leftneighboursizebox.left)/(leftneighboursizebox.right-leftneighboursizebox.left);
		double newtexcenter = leftneighbourtexbox.left + cfac1 * (leftneighbourtexbox.right - leftneighbourtexbox.left);

		//double newtexcenter = leftneighbourtexbox.left + newcenter * (leftneighbourtexbox.right - leftneighbourtexbox.left);

		/*double wfac1 = (newwidth-leftneighboursizebox.left)/(leftneighboursizebox.right-leftneighboursizebox.left);
		double newtexwidth = leftneighbourtexbox.left + wfac1 * (leftneighbourtexbox.right - leftneighbourtexbox.left);
		*/
		double newtexwidth = newwidth * (leftneighbourtexbox.right - leftneighbourtexbox.left);


		MyRect rightneighboursizebox;
		//insert new fold and new plane after leftneighbour
		
		MyRect rightneighbourtexbox;

		//right end of new plane
		rectComponent* nextneighbour = getNextNeighbourById(currentcomponent->getId());
		double newend;
		double newendtex;
		if(nextneighbour == 0)
		{
			newend = 1;
			newendtex = 1;
		}
		else
		{
			newend = nextneighbour->getBoundingBox().left;
			newendtex = nextneighbour->getTextureBoundingBox().left;
		}

		rectComponent* previousneighbour = getPreviousNeighbourById(currentcomponent->getId());
		double newbegin;
		double newbegintex;
		if(previousneighbour == 0)
		{
			newbegin = 0;
			newbegintex = 0;
		}
		else
		{
			newbegin = previousneighbour->getBoundingBox().right;
			newbegintex = previousneighbour->getTextureBoundingBox().right;
		}

		//resize old component
		leftneighboursizebox.left = currentcomponent->getBoundingBox().left;
		leftneighboursizebox.top = currentcomponent->getBoundingBox().top;
		leftneighboursizebox.right = newcenter-newwidth/2;
		leftneighboursizebox.bottom = currentcomponent->getBoundingBox().bottom;
		if(leftneighboursizebox.right < leftneighboursizebox.left) {newStatus(text_status_newfoldfail);return; } //no room for fold, abort


		//create new right neighbour plane
		componentPlane* newplane = new componentPlane(runningId);
		runningId++;	//update the running Id
		MyRect baserect;
		baserect.bottom = currentcomponent->getBoundingBox().bottom;
		baserect.top = currentcomponent->getBoundingBox().top;
		baserect.left = newcenter+newwidth/2;
		baserect.right = newend;
		if(baserect.left > baserect.right) { newStatus(text_status_newfoldfail);return;} //no room for fold, abort
		MyRect baserecttex;
		baserecttex.bottom = baserect.bottom;
		baserecttex.top = baserect.top;
		baserecttex.left = newtexcenter+newtexwidth/2;
		baserecttex.right = newendtex;

		newplane->startUp(baserect,baserecttex);

		//create new fold
		componentFold* newfold = new componentFold(runningId);
		runningId++;	//update the running Id
		MyRect foldrect;
		foldrect.bottom = currentcomponent->getBoundingBox().bottom;
		foldrect.top = currentcomponent->getBoundingBox().top;
		foldrect.left = newcenter-newwidth/2;
		foldrect.right = newcenter+newwidth/2;
		MyRect foldtex;
		foldtex.bottom = foldrect.bottom;
		foldtex.top = foldrect.top;
		foldtex.left = newtexcenter-newtexwidth/2;
		foldtex.right = newtexcenter+newtexwidth/2;
		newfold->startUp(foldrect,foldtex);

		leftneighbourtexbox.right = newfold->getTextureBoundingBox().left;
		leftneighbour->resize(leftneighboursizebox,leftneighbourtexbox);
		//leftneighbourtexbox.left = leftneighbourtexbox.left;
		
		//insert new fold and new plane
		insertComponentAfterId(newfold,leftneighbour->getId());
		insertComponentAfterId(newplane,newfold->getId());
		calcTexCoordsTotal();

		newStatus(text_status_newfold);


	}
	//ELSE IF FOLD
	else if(currentcomponent->getType() == MyCompType::OURFOLD)
	{
		//do nothing on click
	}
	//ELSE IF ??
	else
	{
		//maybe multifold?
	}


}


//This function is supposed to pass drag information on to a fold. 
//Always updateOnClick happens before this
//Should do nothing in a plane (because the click already makes a new fold if clicked on a plane)
void imageRect::updateOnDrag(double xStart, double yStart, double xDiff, double yDiff)
{
	if(dragSelection==-1){return;}
	rectComponent* currentcomponent = getComponentById(dragSelection);
	
	//IF PLANE
	if(currentcomponent->getType() == MyCompType::OURPLANE)
	{



	}
	//ELSE IF FOLD
	else if(currentcomponent->getType() == MyCompType::OURFOLD)
	{
		//resize fold: translate the outer neighbours inwards, translate this borders outwards and resize the fold
		

		//INIT VARIABLES
		//==============
		double dbx = abs(myBoundingBox.right - myBoundingBox.left);
		double dby = abs(myBoundingBox.top - myBoundingBox.bottom);

		
		

		double resizeamount = ((yDiff>0)?val_fold_resizespeed_base:val_fold_resizespeed_base*(-1)) //base positive or negative according to yDiff
			+ (yDiff * val_fold_resizespeed_multiplier)*(1-val_fold_resizespeed_base);

		//float dbx = myBoundingBox.left + (resizeamount * xdiff);

		//float newdbx = abs((myBoundingBox.right - (resizeamount)*dbx) - (myBoundingBox.left + (resizeamount)*dbx));

		//float newresizeamount = (resizeamount/dbx) * newdbx;

		double oldleft = myBoundingBox.left;
		double oldright = myBoundingBox.right;



		double z = ((componentFold*)currentcomponent)->getExplicitDepth();
		double p = (val_fold_start_width*dbx)/2;
		double c = resizeamount;

		double new_z = sqrt( pow( sqrt(pow(p,2) + pow(z,2)) + c , 2 ) - pow(p,2) );

		double resizeCompsAmount = resizeamount;
		double normResizeAmount = resizeamount;
		//double resizeCompsAmount = resizeamount/((myBoundingBox.right - (resizeamount))-(myBoundingBox.left + (resizeamount))); 
		


		rectComponent* leftneighbour = getPreviousNeighbourById(currentcomponent->getId());
		rectComponent* rightneighbour = getNextNeighbourById(currentcomponent->getId());

		MyRect leftneighbourbox = leftneighbour->getBoundingBox();
		MyRect rightneighbourbox = rightneighbour->getBoundingBox();

		MyRect currentcomponentbox = currentcomponent->getBoundingBox();

		//========
		//END INIT
		






		//CHECK BREAK CONDITIONS
		//======================
		

		//this is error
		/*if((myBoundingBox.left + (resizeamount) > currentcomponent->getBoundingBox().left)
		 ||(myBoundingBox.right - (resizeamount) < currentcomponent->getBoundingBox().right))
		{
			newStatus(text_status_resizeerrorsizeedge);
			return; //if the fold sticks out of the rect, cancel
		}*/

		double leftcheck = leftneighbourbox.right;
		double rightcheck = rightneighbourbox.left;

		double zl = addByWorld(leftcheck,resizeamount);
		double zr = addByWorld(rightcheck,resizeamount);

		double fl = subtractByWorld(zl,resizeamount);
		double fr = subtractByWorld(zr,resizeamount);

		if((fl < leftneighbourbox.left)
		 ||(fr > rightneighbourbox.right))
		{
			newStatus(text_status_resizeerrorsizeneighbour);
			return; //if one of the neighbours vanishes, cancel
		}

		


		//==============
		//END CONDITIONS
		









		//DO THE RESIZING
		//===============


		if(currentcomponent->updateFoldDepth(new_z))
		{
			//if this returns true, delete current fold since it has degenerated into a plane.

			deleteFoldWithId(currentcomponent->getId());
			releaseSelection();
			newStatus(text_status_deletefold);
			return;
		}




		//resize outer borders
		myBoundingBox.left = myBoundingBox.left + (resizeamount);	//translate to right
		myBoundingBox.right = myBoundingBox.right - (resizeamount); //translate to left

		

		_m_resizeComps(leftneighbourbox,rightneighbourbox,currentcomponentbox,leftneighbour,rightneighbour,currentcomponent,
			resizeCompsAmount, normResizeAmount);

		

		//============
		//END RESIZING
		


		calcTexCoordsTotal();
		newStatus(text_status_resize);



	}
	//ELSE IF ??
	else
	{
		//maybe multifold?
	}
}

void imageRect::insertComponentAfterId(rectComponent* newcomponent, int id)
{
	rectComponent* currentcomponent=0;

	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size()-1; i++) //only iterate once less than maximum, to find if its the last one
	{
		currentcomponent = (*it);
		if(currentcomponent->getId() == id)
		{
			it++;	//go once further because .insert inserts before the iterator, but we want after
			components.insert(it,newcomponent);
			return;
		}

		it++;
	}

	//if we are still running, it is the last component and we insert at end
	components.push_back(newcomponent);

}
//function deleteComponent

void imageRect::resetComponents()
{
	if(!initialized)
	{
		return;
	}

	runningId=0;
	components.clear();

	myBoundingBox = initBoundingBox;

	componentPlane* baseplane = new componentPlane(runningId);
	runningId++;	//update the running Id
	MyRect baserect;
	baserect.bottom = 0;
	baserect.top = 1;
	baserect.left = 0;
	baserect.right = 1;
	baseplane->startUp(baserect,baserect);

	components.push_back(baseplane);

	calcTexCoordsTotal();

	newStatus(text_status_reset);
}

rectComponent* imageRect::getNextNeighbourById(int id)
{
	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size()-1; i++)
	{
		if( (*it)->getId() == id )	//check all but last component, return the next neighbor
		{
			it++;	
			return (*it);
		}
		it++;
	}
	return 0;	//this is the last component
}

rectComponent* imageRect::getPreviousNeighbourById(int id)
{
	std::list<rectComponent*>::iterator it = components.begin();
	it++;
	for(int i=0; i<components.size()-1; i++)
	{
		if( (*it)->getId() == id )	//check all but first component, return the previous neighbor
		{
			it--;	
			return (*it);
		}
		it++;
	}
	return 0;	//this is the first component
}

void imageRect::newStatus(char* message)
{
	status=message;
}

void imageRect::deleteFoldWithId(int id)
{
	//find the fold, get its next neighbour, delete both and set the previous neighbour to span the area

	rectComponent* nextneighbour = getNextNeighbourById(id);
	MyRect nextneighbourbox = nextneighbour->getBoundingBox();
	MyRect nextneighbourtexbox = nextneighbour->getTextureBoundingBox();
	rectComponent* previousneighbour = getPreviousNeighbourById(id);
	MyRect previousneighbourbox = previousneighbour->getBoundingBox();
	MyRect previousneighbourtexbox = previousneighbour->getTextureBoundingBox();
	previousneighbourbox.right = nextneighbourbox.right;
	previousneighbourtexbox.right = nextneighbourtexbox.right;
	previousneighbour->resize(previousneighbourbox,previousneighbourtexbox);

	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size(); i++)
	{
		if( (*it)->getId() == id )	//found the current fold
		{

			it++;
			nextneighbourbox = (*it)->getBoundingBox();
			components.erase(it);	


			std::list<rectComponent*>::iterator it2 = components.begin();
			for(int i=0; i<components.size(); i++)
			{
				if( (*it2)->getId() == id )	
				{
				
					components.erase(it2);

					calcTexCoordsTotal();
					return;
				}
				it2++;
			}
			
		}
		it++;
	}
}

int imageRect::getNumberOfComponents()
{
	return components.size();
}

double imageRect::addByWorld(double normPos, double worldAmount)
{
	double res=0;

	double port = normPos;
	double invport = 1-normPos;

	//transform normPos into world
	double worldPos = myBoundingBox.left + normPos * abs(myBoundingBox.right-myBoundingBox.left);

	double newWorldPos = 0;
	/*
	if(worldAmount>0)
	{
		//add world Amount
		newWorldPos = worldPos + port*worldAmount;
	}
	else
	{
		newWorldPos = worldPos + invport*worldAmount;
	}
	*/

	newWorldPos = worldPos + port*worldAmount;

	//transform back into norm
	res = (newWorldPos - myBoundingBox.left)/abs(myBoundingBox.right-myBoundingBox.left);

	return res;
}

double imageRect::subtractByWorld(double normPos, double worldAmount)
{
	double res=0;

	double port = normPos;
	double invport = 1-normPos;

	//transform normPos into world
	double worldPos = myBoundingBox.left + normPos * abs(myBoundingBox.right-myBoundingBox.left);

	double newWorldPos = 0;
	/*
	if(worldAmount>0)
	{
		//add world Amount
		newWorldPos = worldPos - port*worldAmount;
	}
	else
	{
		newWorldPos = worldPos - invport*worldAmount;
	}
	*/

	newWorldPos = worldPos - invport*worldAmount;

	//transform back into norm
	res = (newWorldPos - myBoundingBox.left)/abs(myBoundingBox.right-myBoundingBox.left);

	return res;
}

void imageRect::_m_resizeComps(MyRect leftneighbourbox, MyRect rightneighbourbox, MyRect currentcomponentbox, rectComponent* leftneighbour, rectComponent* rightneighbour, rectComponent* currentcomponent, double resizeamount, double normResizeAmount)
{
	MyRect leftntexb = leftneighbour->getTextureBoundingBox();
	MyRect rightntexb = rightneighbour->getTextureBoundingBox();
	MyRect curntexb = currentcomponent->getTextureBoundingBox();

	/*leftntexb.right = leftntexb.right - (1-leftntexb.right)*normResizeAmount;
	rightntexb.left = rightntexb.left + rightntexb.left*normResizeAmount;
	curntexb.left = curntexb.left - (1-curntexb.left)*normResizeAmount;
	curntexb.right = curntexb.right + curntexb.right*normResizeAmount;*/

	//resize neighbours
	//first add the scaling from the right, then subtract the scaling from the left.
	leftneighbourbox.right = addByWorld(leftneighbourbox.right,resizeamount);
	rightneighbourbox.left = addByWorld(rightneighbourbox.left,resizeamount);
	leftneighbourbox.right = subtractByWorld(leftneighbourbox.right,resizeamount);
	rightneighbourbox.left = subtractByWorld(rightneighbourbox.left,resizeamount);


	
	//change the tex coords: shift them inwards by the same distance as the borders (normalized)
	/*leftntexb.right = leftntexb.right - normResizeAmount;
	rightntexb.left = rightntexb.left + normResizeAmount;
	curntexb.left = curntexb.left - normResizeAmount;
	curntexb.right = curntexb.right + normResizeAmount;
	*/

	leftntexb.right = subtractByWorld(leftntexb.right,normResizeAmount);
	rightntexb.left = addByWorld(rightntexb.left,normResizeAmount);
	curntexb.left = subtractByWorld(curntexb.left,normResizeAmount);
	curntexb.right = addByWorld(curntexb.right,normResizeAmount);



	leftneighbour->resize(leftneighbourbox,leftneighbourbox);
	rightneighbour->resize(rightneighbourbox,rightneighbourbox);

	//resize current component
	currentcomponentbox.left = addByWorld(currentcomponentbox.left,resizeamount);
	currentcomponentbox.right = addByWorld(currentcomponentbox.right,resizeamount);
	currentcomponentbox.left = subtractByWorld(currentcomponentbox.left,resizeamount);
	currentcomponentbox.right = subtractByWorld(currentcomponentbox.right,resizeamount);
	currentcomponent->resize(currentcomponentbox,currentcomponentbox);
}

void imageRect::calcTexCoordsTotal()
{
	double cumulativeDistance=0;
	double currentDistance=0;

	//left to right
	std::list<rectComponent*>::iterator it = components.begin();
	for(int i=0; i<components.size(); i++)
	{
		if((*it)->getType() == OURPLANE)	//plane = 4 vertices
		{
			MyRect currentNormPosBound = (*it)->getBoundingBox();
			double leftWorldPos = getWorldPosition(currentNormPosBound.left);
			double rightWorldPos = getWorldPosition(currentNormPosBound.right);

			//set the left coordinate = accumulated distance (or 0)
			double leftTexCoord = cumulativeDistance/initRange;

			//calc the euclidean distance to the right coordinate
			double euclideanDistance = sqrt(pow(rightWorldPos - leftWorldPos,2));

			//accumulate the distance
			currentDistance = cumulativeDistance + euclideanDistance;

			//set the right coordinate
			double rightTexCoord = currentDistance/initRange;

			((componentPlane*)(*it))->setExplicitTexCoord(leftTexCoord,rightTexCoord);

			//carry over accumulated distance
			cumulativeDistance = currentDistance;
		}
		else if ( (*it)->getType() == OURFOLD )		//fold = 6 vertices
		{
			std::vector<std::vector<double>> verticesNorm = (*it)->getPositions();
			//vector contains 3 vectors: xNorm,yNorm,zNorm
			//vertices are: topleft,bottomleft,topmiddle,bottommiddle,topright,bottomright

			double leftXnorm = verticesNorm.at(0).at(0);		//topleft vertex
			double leftXworld = getWorldPosition(leftXnorm);

			double middleXnorm = verticesNorm.at(0).at(2);		//topmiddle vertex
			double middleXworld = getWorldPosition(middleXnorm);
			double middleZworld = verticesNorm.at(2).at(2);		//z is in world space already

			double rightXnorm = verticesNorm.at(0).at(4);		//topright vertex
			double rightXworld = getWorldPosition(rightXnorm);

			
			//distance between topleft and topmiddle
			double firstEuclideanDistance = sqrt(pow(middleXworld - leftXworld,2) + pow(middleZworld,2));

			//distance between topmiddle and topright
			double secondEuclideanDistance = sqrt(pow(rightXworld - middleXworld,2) + pow(-middleZworld,2));


			double leftTexCoord = cumulativeDistance/initRange;

			cumulativeDistance += firstEuclideanDistance;

			double middleTexCoord = cumulativeDistance/initRange;

			cumulativeDistance += secondEuclideanDistance;

			double rightTexCoord = cumulativeDistance/initRange;

			((componentFold*)(*it))->setExplicitTexCoord(leftTexCoord,middleTexCoord,rightTexCoord);


		}

		it++;
	}



}