#ifndef _IMAGERECT
#define _IMAGERECT

#include <QString>
#include <list>
#include <rectComponent.h>
#include <componentPlane.h>
#include <componentFold.h>
#include <QtGui>
#include <QGLWidget>
#include <GLU.H>
#include "config.h"

//this class is the rectangle that represents the image in the work view
//it consists of a chain of planes and folds (from left to right)
//the user can create new folds or resize old folds
class imageRect
{
public:
	imageRect(){ initialized = false; runningId = 0; }
	bool isInitialized(){ return initialized; }
	void startUp( QString filename, double width, double height );
	double* getPositions();
	double* getTexCoords();
	void updateSelection(double x, double y);
	MyRect getBoundingBox() {return myBoundingBox;}
	MyRect getSelectionBox();
	QPixmap* getTex() {return tex;}
	int getVertexCount() {return vertexCount;}		//this is updated in getPositions(dirty)
	const int getCurrentSelectionId() {return selection; }
	char* getStatusMessage() {return status; }
	int getNumberOfComponents();
	void fixSelecction() {dragSelection = selection;}
	void releaseSelection() {dragSelection = -1;}

	void calcTexCoordsTotal();
	void updateOnClick(double x, double y);
	void updateOnDrag(double xStart, double yStart, double xDiff, double yDiff);
	
	void resetComponents();
	void setstartwidth(double width) {val_fold_start_width = width;}

private:
	rectComponent* getComponentById(int id);
	rectComponent* getComponentByPosition(double x, double y);
	void insertComponentAfterId(rectComponent* newComponent, int id);
	rectComponent* getNextNeighbourById(int id);
	rectComponent* getPreviousNeighbourById(int id);
	void deleteFoldWithId(int id);
	double addByWorld(double normPos, double worldAmount);
	double subtractByWorld(double normPos, double worldAmount);
	double getWorldPosition(double xNorm);

	void newStatus(char* message);

	bool initialized;
	std::list<rectComponent*> components;
	int selection;
	int lastSelection;
	int dragSelection;
	MyRect currentSelectionBox;
	MyRect myBoundingBox;
	MyRect initBoundingBox;
	int vertexCount;
	double initRange;
	

	int runningId; //keep an id for all components, increment this when a component is inserted.

	QPixmap *tex;

	char* status;

	double val_fold_start_width;

	//helper
	void _m_resizeComps(MyRect leftneighbourbox, MyRect rightneighbourbox, MyRect currentcomponentbox, rectComponent* leftneighbour, rectComponent* rightneighbour, rectComponent* currentcomponent, double resizeamount, double normResizeAmount); 
};



#endif
