#include "rectComponent.h"

//returns the rect-space bounding box
MyRect rectComponent::getBoundingBox()
{
	return myBoundingBox;
}

MyRect rectComponent::getTextureBoundingBox()
{
	return myTextureBoundingBox;
}


//returns the positions - they are in normalized space (0,1) 
//returns the x,y,z components separately so they can be transformed into world space
std::vector<std::vector<double>> rectComponent::getPositions(){
	std::vector<double> resultX;
	std::vector<double> resultY;
	std::vector<double> resultZ;
	
	std::list<Vertex>::iterator it = vertices.begin();
	for(int i=0; i<vertices.size(); i++){
		resultX.push_back(it->position[0]);
		resultY.push_back(it->position[1]);
		resultZ.push_back(it->position[2]);
		it++;
	}
	
	std::vector<std::vector<double>> resultvec;
	
	resultvec.push_back(resultX);
	resultvec.push_back(resultY);
	resultvec.push_back(resultZ);
	
	return resultvec;
}

std::vector<double> rectComponent::getTexCoords(){
	double* result = new double[vertices.size()*2];
	std::list<Vertex>::iterator it = vertices.begin();
	for(int i=0; i<vertices.size()*2; i++){
		result[i] = it->texCoord[0];
		result[++i] = it->texCoord[1];
		it++;
	}
	std::vector<double> resultvec;
	resultvec.assign(result,result+vertices.size()*2);
	return resultvec;
}

void rectComponent::updateTexCoords(MyRect newTextureBoundingBox)
{
	MyRect oldboundingbox = myTextureBoundingBox;

	std::list<Vertex>::iterator it = vertices.begin();
	for(it; it!=vertices.end(); it++){
		it->texCoord[0]=newTextureBoundingBox.left + (((it->texCoord[0]-oldboundingbox.left)/(oldboundingbox.right-oldboundingbox.left)) * (newTextureBoundingBox.right-newTextureBoundingBox.left)); 	
		it->texCoord[1]=newTextureBoundingBox.bottom +(((it->texCoord[1]-oldboundingbox.bottom)/(oldboundingbox.top-oldboundingbox.bottom)) * (newTextureBoundingBox.top-newTextureBoundingBox.bottom)); 
	}

	myTextureBoundingBox = newTextureBoundingBox;
}

void rectComponent::resize(MyRect newSizeBox, MyRect newTextureBoundingBox)
{
	MyRect oldboundingbox = myBoundingBox;

	//resize all vertices within and the bounding box
	std::list<Vertex>::iterator it = vertices.begin();
	for(it; it!=vertices.end(); it++){
		//x coordinate
		it->position[0] = newSizeBox.left + (((it->position[0]-oldboundingbox.left)/(oldboundingbox.right-oldboundingbox.left)) * (newSizeBox.right-newSizeBox.left)); 
		//y coordinate
		it->position[1] = newSizeBox.bottom +(((it->position[1]-oldboundingbox.bottom)/(oldboundingbox.top-oldboundingbox.bottom)) * (newSizeBox.top-newSizeBox.bottom)); 
	}

	//set new bounding box
	myBoundingBox = newSizeBox;

	//update textures
	updateTexCoords(newTextureBoundingBox);
}