#include <QtGui>

#include <qboxlayout.h>
#include <qfiledialog.h>

#include "glwidget.h"
#include "window.h"

Window::Window()
{
	QGridLayout *mainLayout = new QGridLayout;

	QColor clearColor;
	clearColor.setRgb(MyGL_BACKGROUNDCOLOR_r,MyGL_BACKGROUNDCOLOR_g,MyGL_BACKGROUNDCOLOR_b);

	glWidget = new GLWidget(0, 0);
	glWidget->setClearColor(clearColor);
	//glWidget->rotateBy(+42 * 16, +42 * 16, -21 * 16);
	mainLayout->addWidget(glWidget, 0, 0);

	QVBoxLayout* toolpanellayout = new QVBoxLayout;
	

	openfilebutton = new QPushButton(0);
	openfilebutton->setText(QString(text_button_fopenbutton));
	toolpanellayout->addWidget(openfilebutton);
	QObject::connect(openfilebutton, SIGNAL(released()),
                      this, SLOT(openFile()));

	openfilelabel = new QLabel(0);
	openfilelabel->setText(QString(text_label_fopenlabel));
	toolpanellayout->addWidget(openfilelabel);

	optionselectionbox = new QCheckBox(QString(text_label_checkboxselectionbox),0);
	optionselectionbox->setChecked(true);
	toolpanellayout->addWidget(optionselectionbox);
	QObject::connect(optionselectionbox, SIGNAL(stateChanged(int)),
                      this, SLOT(setSelectionBoxEnabled(int)));

	selectedcomponentlabel = new QLabel(0);
	selectedcomponentlabel->setText(QString(text_label_noselectionidlabel));
	QObject::connect(glWidget, SIGNAL(idChanged(int)),
                      this, SLOT(setSelectedComponentLabelId(int)));
	toolpanellayout->addWidget(selectedcomponentlabel);

	componentnumberlabel = new QLabel(0);
	componentnumberlabel->setText(QString(text_label_numbercomponentlabel) + QString::number(0));
	QObject::connect(glWidget, SIGNAL(updateNumberOfComponents(int)),
                      this, SLOT(setNumberOfComponents(int)));
	toolpanellayout->addWidget(componentnumberlabel);
	

	resetbutton = new QPushButton(0);
	resetbutton->setText(QString(text_button_resetbuttonlabel));
	toolpanellayout->addWidget(resetbutton);
	QObject::connect(resetbutton, SIGNAL(released()),
                      this, SLOT(resetComponents()));

	configeditlabel = new QLabel(0);
	configeditlabel->setText(QString(text_label_configeditlabel));
	toolpanellayout->addWidget(configeditlabel);

	configedit = new QLineEdit(0);
	configedit->setText(QString::number(Config::val_fold_startval_width));
	toolpanellayout->addWidget(configedit);
	QObject::connect(configedit, SIGNAL(textChanged(QString)),
                      this, SLOT(setComponentWidth(QString)));


	toolpanellayout->addStretch();

	statuslabel = new QLabel(0);
	statuslabel->setText(QString(text_status_statuslabel)+QString(text_status_start));
	QObject::connect(glWidget, SIGNAL(statusChanged(QString)),
                      this, SLOT(changeStatus(QString)));
	toolpanellayout->addWidget(statuslabel);

	mainLayout->addLayout(toolpanellayout,0,1);

	setLayout(mainLayout);


	//QTimer *timer = new QTimer(this);
	//connect(timer, SIGNAL(timeout()), this, SLOT(rotateOneStep()));
	//timer->start(20);

	setWindowTitle(QString(text_title_mainwindow));
}

void Window::openFile()
{
	 QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                     "",
                                                     tr("Files (*.*)"));
	 QString name = fileName.split("/").last();
	 openfilelabel->setText(QString(text_label_fopenedlabel)+name);

	 glWidget->loadTexture(fileName);

}

void Window::setSelectionBoxEnabled(int enabled)
{
	bool res = false;
	if(enabled==2) {res=true;}
	glWidget->setSelectionBox(res);
}

void Window::setSelectedComponentLabelId(int id)
{
	if(id==-1)
	{
		selectedcomponentlabel->setText(QString(text_label_noselectionidlabel));
	}
	else
	{
		selectedcomponentlabel->setText(QString(text_label_selectionidlabel)+QString::number(id));
	}
}

void Window::resetComponents()
{
	glWidget->resetComponents();
}

void Window::changeStatus(QString message)
{
	statuslabel->setText(QString(text_status_statuslabel)+message);
}

void Window::setNumberOfComponents(int count)
{
	componentnumberlabel->setText(QString(text_label_numbercomponentlabel)+QString::number(count));
}

void Window::setComponentWidth(QString numberstring)
{
	QLocale c(QLocale::C);
	double d = c.toDouble(numberstring);
	glWidget->setstartwidth(d);
}